<?php

namespace Bixo\Request\Http\Controllers;

use App\Http\Controllers\ActionController as BaseController;
use Exception;
use Bixo\Request\Actions\RequestAction;
use Bixo\Request\Http\Requests\RequestActionRequest;
use Bixo\Request\Http\Resources\RequestResource;
use Bixo\Request\Models\Request;

/**
 * Admin web controller class.
 */
class RequestActionController extends BaseController
{

    /**
     * Action controller function for request.
     *
     * @param Model $model
     * @param action next action for the model.
     *
     * @return Response
     */
    public function __invoke(RequestActionRequest $request, Request $request, $action)
    {
        try {
            $request = $request->all();
            $request = RequestAction::run($action, $request, $request);

            $data = new RequestResource($request);
            return $this->response->message(trans("messages.success.{$action}", ['Module' => trans('request::request.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('request/request/' . $request->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('request/request/' . $request->getRouteKey()))
                ->redirect();
        }
    }

    public function notify(){
        
    }
}
