<?php

namespace Bixo\Request\Http\Resources;

use Illuminate\Http\Resources\Json\JsonResource;

class RequestResource extends JsonResource
{

    public function itemLink()
    {
        return guard_url('request/request') . '/' . $this->getRouteKey();
    }

    public function title()
    {
        if ($this->title != '') {
            return $this->title;
        }

        if ($this->name != '') {
            return $this->name;
        }

        return 'None';
    }

    public function toArray($request)
    {
        return [
            'id' => $this->getRouteKey(),
            'title' => $this->title(),
            'slug' => $this->slug,
            'ref' => $this->ref,
            'employee_id' => $this->employee_id,
            'handover_to' => $this->handover_to,
            'handover_approved_id' => $this->handover_approved_id,
            'team_id' => $this->team_id,
            'accounts_id' => $this->accounts_id,
            'mobile' => $this->mobile,
            'email' => $this->email,
            'type' => $this->type,
            'title' => $this->title,
            'date_from' => $this->date_from,
            'date_to' => $this->date_to,
            'period' => $this->period,
            'document_type' => $this->document_type,
            'addresee' => $this->addresee,
            'leave_type' => $this->leave_type,
            'leave_paid' => $this->leave_paid,
            'leave_half_day' => $this->leave_half_day,
            'paid' => $this->paid,
            'unpaid' => $this->unpaid,
            'description' => $this->description,
            'notes' => $this->notes,
            'amount' => $this->amount,
            'status' => $this->status,
            'approved_id' => $this->approved_id,
            'approved_at' => $this->approved_at,
            'rejected_id' => $this->rejected_id,
            'rejected_at' => $this->rejected_at,
            'manager_id' => $this->manager_id,
            'manager_status' => $this->manager_status,
            'hr_id' => $this->hr_id,
            'hr_status' => $this->hr_status,
            'rejected_reason' => $this->rejected_reason,
            'rejected_description' => $this->rejected_description,
            'accepted_id' => $this->accepted_id,
            'accepted_at' => $this->accepted_at,
            'hr_approved_at' => $this->hr_approved_at,
            'return_to' => $this->return_to,
            'salary' => $this->salary,
            'add_commission' => $this->add_commission,
            'traveling_to' => $this->traveling_to,
            'user_id' => $this->user_id,
            'user_type' => $this->user_type,
            'created_at' => !is_null($this->created_at) ? $this->created_at->format('Y-m-d H:i:s') : null,
            'updated_at' => !is_null($this->updated_at) ? $this->updated_at->format('Y-m-d H:i:s') : null,
            'meta' => [
                'exists' => $this->exists(),
                'link' => $this->itemLink(),
                'upload_url' => $this->getUploadURL(''),
                'workflow' => $this->workflows(),
                'actions' => $this->actions(),
            ],
        ];
    }

    /**
     * Get additional data that should be returned with the resource array.
     *
     * @param     \Illuminate\Http\Request  $request
     * @return array
     */
    public function with($request)
    {
        return [
            'meta' => [
                'exists' => $this->exists(),
                'link' => $this->itemLink(),
                'upload_url' => $this->getUploadURL(''),
                'workflow' => $this->workflows(),
                'actions' => $this->actions(),
            ],
        ];
    }

    /**
     * Get the workflows for the resource.
     *
     * @return array
     */
    private function workflows()
    {
        $arr = [];
                $workflow = $this->resource->workflow();
        foreach ($workflow->transitions($this) as $key => $transition) {
            $name = $transition->getName();
            $arr[$key]['url'] = guard_url('request/request/workflow/' . $this->getRouteKey() . '/' . $name);
            $arr[$key]['name'] = $name;
            $arr[$key]['key'] = $name;
            $arr[$key]['form'] = $workflow->form($transition);
            $arr[$key]['label'] = trans('request::request.workflow.' . $name);
        }
                return $arr;

    }
    
    /**
     * Get the actions for the resource.
     *
     * @return array
     */
    private function actions()
    {

        $arr = [];
                $actions = $this->resource->actions()->details();
        foreach ($actions as $key => $action) {
            $name = $action->name();
            $arr[$key]['url'] = guard_url('request/request//action' . $this->getRouteKey() . '/' . $action->name());
            $arr[$key]['name'] = $name;
            $arr[$key]['key'] = $name;
            $arr[$key]['form'] = $action->form();
            $arr[$key]['label'] = trans('request::request.actions.' . $name);
        }
        
        return $arr;
    }
}
