<?php

namespace Bixo\Request\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Bixo\Request\Events\RequestAction as RequestActionEvent;
use Bixo\Request\Notifications\RequestAction as RequestActionNotification;
use Litepie\Actions\Concerns\AsAction;

class RequestAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the RequestActionEvent.
     *
     * @param   RequestActionEvent  $event
     * @return mixed
     */
    public function handle(RequestActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $request.
     *
     * @param   RequestActionEvent  $event
     * @return void
     */
    public function create(RequestActionEvent $event)
    {
        $client = $event->request->client;
        Notification::send($client, new RequestActionNotification($event));
    }

    /**
     * Handle the RequestActionEvent as a listener.
     *
     * @param   RequestActionEvent  $event
     * @return mixed
     */
    public function asListener(RequestActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   RequestActionEvent  $event
     * @return bool
     */
    private function isAllowed(RequestActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
