<?php

namespace Bixo\Request\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Bixo\Request\Events\RequestWorkflow as RequestWorkflowEvent;
use Bixo\Request\Notifications\RequestWorkflow as RequestWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class RequestWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(RequestWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $request is submitted.
     *
     * @param RequestWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(RequestWorkflowEvent $event)
    {
        $client = $event->request->client;
        Notification::send($client, new RequestWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $request is published.
     *
     * @param RequestWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(RequestWorkflowEvent $event)
    {
        $client = $event->request->client;
        Notification::send($client, new RequestWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $request is approved.
     *
     * @param RequestWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(RequestWorkflowEvent $event)
    {
        $client = $event->request->client;
        Notification::send($client, new RequestWorkflowNotification($event));
    }

    /**
     * Handles the $request workflow event as a listener.
     *
     * @param RequestWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(RequestWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
