<?php

namespace Bixo\Request\Policies;

use Illuminate\Foundation\Auth\User as Authenticatable;
use Bixo\Request\Models\Request;

class RequestPolicy
{

    use RequestAction;
    use RequestReport;
    use RequestWorkflow;

    /**
     * Determine if the given user can view the request.
     *
     * @param Authenticatable $user
     * @param Request $request
     *
     * @return bool
     */
    public function view(Authenticatable $user, Request $request)
    {
        if ($authUser->canDo('request.request.view') && $authUser->isAdmin() || $user->isClient()) {
            return true;
        }

        return $request->is_owner;
    }

    /**
     * Determine if the given user can create a request.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function create(Authenticatable $user)
    {
        return  $authUser->canDo('request.request.create');
    }

    /**
     * Determine if the given user can update the given request.
     *
     * @param Authenticatable $user
     * @param Request $request
     *
     * @return bool
     */
    public function update(Authenticatable $user, Request $request)
    {
        if ($user->canDo('request.request.edit') && $user->isAdmin()) {
            return true;
        }

        return $request->is_owner;
    }

    /**
     * Determine if the given user can delete the given request.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function destroy(Authenticatable $user, Request $request)
    {
        return $request->is_owner;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
